/**
 * @file dr.h
 *
 * Copyright (C) IBM Corporation 2006
 */

#ifndef _H_DR
#define _H_DR

#include <syslog.h>
#include <signal.h>
#include <nl_types.h>
#include <unistd.h>
#include "rtas_calls.h"
#include "drpci.h"

extern int debug;
extern int debug_fd;

#define dbg(_f, _a...)							\
		do {							\
			if (debug >= 4)	{				\
				char __buf[256];			\
				int __sz;				\
									\
				__sz = sprintf(__buf, "%s(): " _f,	\
					       __FUNCTION__, ##_a);	\
				fprintf(stderr, __buf);			\
				write(debug_fd, __buf, __sz);		\
			}						\
		} while(0)

extern char *cmd;
#define err_msg(_f, _a...)						\
		do {							\
			fprintf(stderr, "%s: " _f, cmd, ##_a);		\
			if (debug >= 4)	{				\
				char __buf[256];			\
				int __sz;				\
									\
				__sz = sprintf(__buf, "%s: " _f,	\
					       cmd, ##_a);		\
				write(debug_fd, __buf, __sz);		\
			}						\
		} while(0)

#define is_dot_dir(_p)	(_p[0] == '.')

#define DR_LOCK_FILE    	"/var/lock/dr_config_lock"
#define PLATFORMPATH    	"/proc/device-tree/device_type"
#define OFDTPATH    		"/proc/ppc64/ofdt"
#define DR_COMMAND		"drslot_chrp_%s"
#define DRMIG_COMMAND		"drmig_chrp_%s"

#define DR_WAIT		-1    /* Wait forever  */
#define DR_NOWAIT	0     /* Return immediately if lock cannot be granted */
#define LOCK_TIMEOUT	5     /* Wait 5 seconds before giving up on dr_lock() */

struct options {
	int     action;	      /* remove, add, REPLACE, IDENTIFY ...           */
#define NONE		0
#define ADD 	     	1
#define REMOVE       	2
#define QUERY		3
#define REPLACE		4
#define IDENTIFY	5
#define MIGRATE		6

	int     no_ident;     /* used in drslot_chrp_pci                      */
	uint    timeout;      /* used in remove operation only. Default
	                       * is 60 seconds. User input in minutes.        */
	char   *usr_drc_name; /* pointer to user-specified drc-name
	                       * of resource                                  */
	int     noprompt;     /* 1 = do not prompt user for input, assume yes */
	int     quantity;     /* number of resources                          */
	char	*ctype;
	char	*p_option;
};

/* The follwing are defined in common.c */
int dr_lock(int);
int dr_unlock(int);
int valid_platform(const char *);
int add_node(char *, struct of_node *);
int process_add_nodes(struct of_node *);
int remove_node(char *);
int add_nodes_to_kernel(char *, struct of_node *);
int remove_nodes_from_kernel(char *);
int get_property(const char *, const char *, void *, size_t);
int get_int_attribute(const char *, const char *, void *, size_t);
int get_str_attribute(const char *, const char *, void *, size_t);
int get_property_size(const char *, const char *);
int signal_handler(int, int, struct sigcontext *);
int mask_signals(void);
char *node_type(struct node *);
int kernel_check(const char *);

void set_debug(int);
void clear_debug();

#define DR_PATH_MAX	1024
#define DR_BUF_SZ	256

int drslot_chrp_slot(struct options *);
int valid_slot_options(struct options *);

int drslot_chrp_cpu(struct options *);
int valid_cpu_options(struct options *);

int drslot_chrp_pci(struct options *);
int valid_pci_options(struct options *);

int drslot_chrp_phb(struct options *);
int valid_phb_options(struct options *);

int drslot_chrp_mem(struct options *);
int valid_mem_options(struct options *);

int drslot_chrp_hea(struct options *);
int valid_hea_options(struct options *);

int drmig_chrp_pmig(struct options *);
int valid_pmig_options(struct options *);

#endif
