/**
 * @file drcpu.h
 *
 * Copyright (C) IBM Corporation 2006
 */

#ifndef _H_DRCPU
#define _H_DRCPU

#include "dr.h"

struct cache_info {
	char		name[DR_BUF_SZ];	/* node name */
	char		path[DR_BUF_SZ];	/* node path */
	struct cache_info *next;		/* global list */
};

/* A "logical" cpu */
struct thread_info {
	int		id;			/* linux "logical" cpu id */
	char		path[DR_BUF_SZ];	/* node path */
	struct thread_info *next;		/* global list */
};

/* A physical cpu that is owned by the partition */
struct cpu_info {
	char		name[DR_BUF_SZ];	/* node name */
	char		path[DR_BUF_SZ];	/* node path */
	struct cpu_info *next;			/* global list */
};

struct dr_info {
	struct cpu_info *all_cpus;
	struct cache_info *all_caches;
	struct thread_info *all_threads;
	struct dr_connector *all_drcs;
};

int cpu_drc_index(struct cpu_info *);
int cache_phandle(struct cache_info *);

int init_cpu_drc_info(struct dr_info *);
void free_cpu_drc_info(struct dr_info *);

struct thread_info * id_to_thread(int, struct dr_info *);
struct thread_info * cpu_get_threads(struct cpu_info *, struct thread_info *);
struct cpu_info * thread_to_cpu(struct thread_info *, struct dr_info *);

int get_thread_state(struct thread_info *);
int set_thread_state(struct thread_info *, int);

int get_cpu_state(struct cpu_info *);
int set_cpu_state(struct cpu_info *, int, struct dr_info *);

int cpu_enable_smt(struct cpu_info *, struct dr_info *);
int cpu_disable_smt(struct cpu_info *, struct dr_info *);

int smt_enabled(struct dr_info *);
int system_enable_smt(struct dr_info *);
int system_disable_smt(struct dr_info *);

int get_cache_use_count(struct cache_info *);

struct cpu_info * drc_index_to_cpu(int, struct dr_info *);
int cpu_drc_index(struct cpu_info *);
char *cpu_drc_name(struct cpu_info *, struct dr_info *);
int cache_phandle(struct cache_info *);

struct cache_info * cpu_get_dependent_cache(struct cpu_info *,
					    struct dr_info *);
struct cache_info * cache_get_dependent_cache(struct cache_info *,
					      struct dr_info *);
int release_cpu(struct cpu_info *, struct dr_info *);
struct cpu_info * acquire_cpu(int, struct dr_info *);

#endif /* _H_DRCPU */
